//==============================================================================
//
//   MainFrame.java
//
//   Copyright (c) 2001-2003 Graffiti Team, Uni Passau
//
//==============================================================================
// $Id: MainFrame.java,v 1.36 2004/04/23 09:03:53 pausch Exp $

package org.graffiti.editor;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.IllegalComponentStateException;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;

import java.beans.PropertyVetoException;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Vector;
import java.util.prefs.BackingStoreException;
import java.util.prefs.InvalidPreferencesFormatException;
import java.util.prefs.Preferences;

import javax.swing.Action;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDesktopPane;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JInternalFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JToolBar;
import javax.swing.KeyStroke;
import javax.swing.ScrollPaneConstants;
import javax.swing.WindowConstants;
import javax.swing.event.InternalFrameAdapter;
import javax.swing.event.InternalFrameEvent;
import javax.swing.event.UndoableEditEvent;
import javax.swing.event.UndoableEditListener;
import javax.swing.undo.UndoableEditSupport;

import org.graffiti.core.ImageBundle;
import org.graffiti.core.StringBundle;

import org.graffiti.editor.actions.CopyAction;
import org.graffiti.editor.actions.CutAction;
import org.graffiti.editor.actions.DeleteAction;
import org.graffiti.editor.actions.EditRedoAction;
import org.graffiti.editor.actions.EditUndoAction;
import org.graffiti.editor.actions.ExitAction;
import org.graffiti.editor.actions.FileCloseAction;
import org.graffiti.editor.actions.FileNewAction;
import org.graffiti.editor.actions.FileOpenAction;
import org.graffiti.editor.actions.FileSaveAction;
import org.graffiti.editor.actions.FileSaveAllAction;
import org.graffiti.editor.actions.FileSaveAsAction;
import org.graffiti.editor.actions.PasteAction;
import org.graffiti.editor.actions.PluginManagerEditAction;
import org.graffiti.editor.actions.RedrawViewAction;
import org.graffiti.editor.actions.RunAlgorithm;
import org.graffiti.editor.actions.SelectAllAction;
import org.graffiti.editor.actions.ViewNewAction;

import org.graffiti.event.ListenerManager;

import org.graffiti.graph.Graph;

import org.graffiti.managers.AlgorithmManager;
import org.graffiti.managers.AttributeComponentManager;
import org.graffiti.managers.DefaultAlgorithmManager;
import org.graffiti.managers.DefaultIOManager;
import org.graffiti.managers.DefaultModeManager;
import org.graffiti.managers.DefaultToolManager;
import org.graffiti.managers.DefaultViewManager;
import org.graffiti.managers.EditComponentManager;
import org.graffiti.managers.IOManager;
import org.graffiti.managers.ModeManager;
import org.graffiti.managers.ToolManager;
import org.graffiti.managers.ViewManager;
import org.graffiti.managers.pluginmgr.PluginDescription;
import org.graffiti.managers.pluginmgr.PluginManager;
import org.graffiti.managers.pluginmgr.PluginManagerException;
import org.graffiti.managers.pluginmgr.PluginManagerListener;

import org.graffiti.plugin.EditorPlugin;
import org.graffiti.plugin.GenericPlugin;
import org.graffiti.plugin.actions.GraffitiAction;
import org.graffiti.plugin.algorithm.Algorithm;
import org.graffiti.plugin.editcomponent.NeedEditComponents;
import org.graffiti.plugin.gui.GraffitiComponent;
import org.graffiti.plugin.gui.GraffitiContainer;
import org.graffiti.plugin.gui.ModeToolbar;
import org.graffiti.plugin.gui.PluginPanel;
import org.graffiti.plugin.gui.ToolButton;
import org.graffiti.plugin.io.InputSerializer;
import org.graffiti.plugin.io.OutputSerializer;
import org.graffiti.plugin.mode.Mode;
import org.graffiti.plugin.tool.AbstractUndoableTool;
import org.graffiti.plugin.tool.Tool;
import org.graffiti.plugin.view.MessageListener;
import org.graffiti.plugin.view.View;
import org.graffiti.plugin.view.ViewListener;

import org.graffiti.selection.Selection;
import org.graffiti.selection.SelectionListener;
import org.graffiti.selection.SelectionModel;

import org.graffiti.session.EditorSession;
import org.graffiti.session.Session;
import org.graffiti.session.SessionListener;
import org.graffiti.session.SessionManager;

import org.graffiti.util.DesktopMenuManager;
import org.graffiti.util.MaximizeManager;
import org.graffiti.util.ToolBarLayout;

/**
 * Constructs a new graffiti frame, which contains the main gui components.
 *
 * @version $Revision: 1.36 $
 */
public class MainFrame
    extends JFrame
    implements SessionManager, SessionListener, PluginManagerListener,
        WindowListener, ComponentListener, MessageListener,
        UndoableEditListener, EditorDefaultValues, IOManager.IOManagerListener,
        ViewManager.ViewManagerListener
{
    //~ Static fields/initializers =============================================

    /** The size of an internal frame for first dispaying. */
    public static final Dimension PREFERRED_INTERNALFRAME_SIZE = new Dimension(1000,
            1000);

    //~ Instance fields ========================================================

    /**
     * The preferences of the editor's main frame. (e.g.: position and size of
     * the main frame.
     */
    protected Preferences uiPrefs;

    /** The current active session. */
    EditorSession activeSession;

    /** Holds all active frames. */
    List activeFrames = new LinkedList();

    /** The list of registered <code>ZoomListener</code>s. */
    List zoomListeners;

    /** Maps from views to internal frames. */
    Map viewFrameMapper;

    /** Contains all <code>Session</code>s. */
    Set sessions = new HashSet();

    /** Handles the algorithms. */
    private AlgorithmManager algorithmManager;

    /** Handles the list of attribute components. */
    private AttributeComponentManager attributeComponentManager;

    /** Handles the list of value edit components. */
    private EditComponentManager editComponentManager;

    /** The main frame's static actions */
    private GraffitiAction editCopy;

    /** The main frame's static actions */
    private GraffitiAction editCut;

    /** The main frame's static actions */
    private GraffitiAction editDelete;

    /** The main frame's static actions */
    private GraffitiAction editPaste;

    /** The main frame's static actions */
    private GraffitiAction editRedo;

    /** The main frame's static actions */
    private GraffitiAction editSelectAll;

    /** The main frame's static actions */
    private GraffitiAction editUndo;

    /** The main frame's static actions */
    private GraffitiAction fileClose;

    /** The main frame's static actions */
    private GraffitiAction fileExit;

    /** The main frame's static actions */
    private GraffitiAction fileOpen;

    /** The main frame's static actions */
    private GraffitiAction fileSave;

    /** The main frame's static actions */
    private GraffitiAction fileSaveAll;

    /** The main frame's static actions */
    private GraffitiAction fileSaveAs;

    /** The main frame's static actions */
    private GraffitiAction newGraph;

    /** The main frame's static actions */
    private GraffitiAction pluginManagerEdit;

    /** The main frame's static actions */
    private GraffitiAction redrawView;

    /** The main frame's static actions */
    private GraffitiAction viewNew;

    /** The listener for the internal frames. */
    private GraffitiInternalFrameListener gift;

    /** The manager for IO serializers. */
    private IOManager ioManager;

    /** The <code>ImageBundle</code> of the main frame. */
    private ImageBundle iBundle = ImageBundle.getInstance();

    /** The desktop pane for the internal frames. */
    private JDesktopPane desktop;

    /** The main frame's menu entries. */
    private JMenu pluginMenu;

    /** DOCUMENT ME! */
    private JMenu windowMenu;

    /** Container for toolbars at the top of the main frame. */
    private JPanel leftToolBarPanel;

    /** Container for toolbars at the left of the main frame. */
    private JPanel topToolBarPanel;

    /** The split pane between the center and the pluginPanel. */
    private JSplitPane vertSplitter;

    /**
     * The list of algorithm actions.
     *
     * @see org.graffiti.editor.actions.RunAlgorithm
     */
    private List algorithmActions;

    /** The list of registered <code>SelectionListener</code>s. */
    private List selectionListeners;

    /** The list of registered <code>SessionListener</code>s. */
    private List sessionListeners;

    /**
     * Contains a mapping between the identifiers of gui-components and the
     * corresponding gui-component.
     */
    private Map guiMap = new HashMap();

    /** The mode manager. */
    private ModeManager modeManager;

    /** A reference to the graffiti plugin manager. */
    private PluginManager pluginmgr;

    /** The panel for the plugins. */
    private PluginPanel pluginPanel;

    /** The main frame's status bar. */
    private StatusBar statusBar;

    /**
     * The default view type, that will be always displayed if the user
     * deactivates the view chooser dialog. This variable is initialized with
     * null per default. for setting the default view this member variable
     * have to be initialized with a valid view type by the method
     * <code>getDefaultView()</code>.
     */
    private String defaultView = null;

    /** The <code>StringBundle</code> of the main frame. */
    private StringBundle sBundle = StringBundle.getInstance();

    /**
     * The listener for the tools. Responsible for changeing the tools when the
     * mode is changed.
     */
    private ToolListener tl;

    /** The tool manager. */
    private ToolManager toolManager;

    /** This object is listener of all undoable actions. */
    private UndoableEditSupport undoSupport;

    /** The manager, which maps view type names to view types. */
    private ViewManager viewManager;

    //~ Constructors ===========================================================

    /**
     * Constructs a new <code>MainFrame</code>.
     *
     * @param pluginmgr DOCUMENT ME!
     * @param prefs DOCUMENT ME!
     */
    public MainFrame(PluginManager pluginmgr, Preferences prefs)
    {
        super();

        this.setTitle(sBundle.getString("name") + " " +
            sBundle.getString("version") + " " +
            sBundle.getString("version.Release") + "." +
            sBundle.getString("version.Major") + "." +
            sBundle.getString("version.Minor"));

        this.sessionListeners = new LinkedList();
        this.selectionListeners = new LinkedList();
        this.zoomListeners = new LinkedList();
        this.viewFrameMapper = new HashMap();
        this.algorithmActions = new LinkedList();
        this.addSessionListener(this);

        viewManager = new DefaultViewManager();
        algorithmManager = new DefaultAlgorithmManager();
        modeManager = new DefaultModeManager();
        toolManager = new DefaultToolManager(modeManager);
        ioManager = new DefaultIOManager();
        attributeComponentManager = new AttributeComponentManager();
        editComponentManager = new EditComponentManager();

        pluginmgr.addPluginManagerListener(viewManager);
        pluginmgr.addPluginManagerListener(toolManager);
        pluginmgr.addPluginManagerListener(algorithmManager);
        pluginmgr.addPluginManagerListener(modeManager);
        pluginmgr.addPluginManagerListener(ioManager);
        pluginmgr.addPluginManagerListener(attributeComponentManager);
        pluginmgr.addPluginManagerListener(editComponentManager);

        ioManager.addListener(this);
        viewManager.addListener(this);

        tl = new ToolListener();

        undoSupport = new UndoableEditSupport();

        //undoSupport.addUndoableEditListener(this);
        gift = new GraffitiInternalFrameListener();

        this.pluginmgr = pluginmgr;

        this.uiPrefs = prefs;

        GraffitiSingleton.getInstance().setMainFrame(this);

        createActions();

        // initialize map of gui components and create menu bar
        guiMap = new Hashtable();

        // create and set the menu bar
        setJMenuBar(createMenuBar());

        // the editor's status bar
        statusBar = new StatusBar(sBundle);
        addSessionListener(statusBar);
        selectionListeners.add(statusBar);
        getContentPane().add(statusBar, BorderLayout.SOUTH);

        // create the desktop
        desktop = new JDesktopPane();
        new MaximizeManager(desktop, getJMenuBar());
        new DesktopMenuManager(desktop, windowMenu);

        // create a panel, which will contain the views for plugins
        pluginPanel = new PluginPanel();
        guiMap.put(pluginPanel.getId(), pluginPanel);

        vertSplitter = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, desktop,
                pluginPanel);

        vertSplitter.setContinuousLayout(true);
        vertSplitter.setOneTouchExpandable(true);
        vertSplitter.setDividerLocation(uiPrefs.getInt("vertSplitter",
                VERT_SPLITTER));

        getContentPane().add(vertSplitter, BorderLayout.CENTER);

        // top toolbars
        topToolBarPanel = new JPanel();
        topToolBarPanel.setLayout(new ToolBarLayout());
        guiMap.put("toolbarPanel", topToolBarPanel);
        getContentPane().add(topToolBarPanel, BorderLayout.NORTH);

        JToolBar toolBar = this.createToolBar();
        guiMap.put("defaultToolbar", toolBar);
        topToolBarPanel.add(toolBar);

        // left toolbars
        leftToolBarPanel = new JPanel();
        leftToolBarPanel.setLayout(new BoxLayout(leftToolBarPanel,
                BoxLayout.Y_AXIS));
        getContentPane().add(leftToolBarPanel, BorderLayout.WEST);

        // window settings like position and size
        setSize(uiPrefs.getInt("sizeWidth", SIZE_WIDTH),
            uiPrefs.getInt("sizeHeight", SIZE_HEIGHT));

        Dimension screenDim = Toolkit.getDefaultToolkit().getScreenSize();
        int xpos = (screenDim.width / 2) - (getWidth() / 2);
        int ypos = (screenDim.height / 2) - (getHeight() / 2);

        setLocation(uiPrefs.getInt("positionX", xpos),
            uiPrefs.getInt("positionY", ypos));

        // just for debugging - rs
        //        System.err.println("entries in the guiMap:");
        //        for (Iterator itr = guiMap.keySet().iterator(); itr.hasNext();) {
        //            System.err.println(itr.next());
        //        }
        // register the main frame to the plugin manager.
        pluginmgr.addPluginManagerListener(this);

        this.addComponentListener(this);

        setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
    }

    //~ Methods ================================================================

    /**
     * Returns the current active editor session.
     *
     * @return the current active editor session.
     */
    public EditorSession getActiveEditorSession()
    {
        return (EditorSession) activeSession;
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public List getActiveFrames()
    {
        return activeFrames;
    }

    /**
     * Returns the current active session.
     *
     * @return the current active session.
     */
    public Session getActiveSession()
    {
        return activeSession;
    }
    
    /**
     * Sets the current active session.
     * @param s The session to be activated.
     */
    public void setActiveSession(Session s) {
        activeSession = (EditorSession)s;
        fireSessionChanged(s);
    }    

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public List getAlgorithmActions()
    {
        return algorithmActions;
    }

    /**
     * Sets the defaultView.
     *
     * @param defaultView The defaultView to set
     */
    public void setDefaultView(String defaultView)
    {
        this.defaultView = defaultView;
    }

    /**
     * Returns the defaultView.
     *
     * @return String
     */
    public String getDefaultView()
    {
        return defaultView;
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public EditComponentManager getEditComponentManager()
    {
        return editComponentManager;
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public IOManager getIoManager()
    {
        return ioManager;
    }

    /**
     * Get the algorithm manager.
     * 
     * @return the algorithm manager.
     */
    public AlgorithmManager getAlgorithmManager() {
    	return algorithmManager;
    }
    
    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public PluginManager getPluginManager()
    {
        return pluginmgr;
    }

    /**
     * Returns <code>true</code>, if a session is active.
     *
     * @return DOCUMENT ME!
     */
    public boolean isSessionActive()
    {
        return getActiveSession() != null;
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public SessionManager getSessionManager()
    {
        return this;
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public Set getSessions()
    {
        return sessions;
    }

    /**
     * Returns an iterator over all sessions.
     *
     * @return an iterator over all sessions.
     *
     * @see org.graffiti.session.Session
     */
    public Iterator getSessionsIterator()
    {
        return sessions.iterator();
    }

    /**
     * DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public Preferences getUiPrefs()
    {
        return uiPrefs;
    }

    /**
     * Returns the class for undoSupport.
     *
     * @return the class for undoSupport.
     */
    public UndoableEditSupport getUndoSupport()
    {
        return undoSupport;
    }

    /**
     * Returns the viewManager.
     *
     * @return ViewManager
     */
    public ViewManager getViewManager()
    {
        return viewManager;
    }

    /**
     * Returns the zoomListeners.
     *
     * @return List
     */
    public List getZoomListeners()
    {
        return zoomListeners;
    }

    /**
     * Adds the <code>JComponent</code> component to the gui-component
     * specified by id. If the specified gui-component does not exist a new
     * one will be created with id id and the newly created gui-component will
     * be added to the gui of the editor.
     *
     * @param id the id of the gui-component where the component shall be
     *        added.
     * @param component the <code>JComponent</code> which shall be added to the
     *        specified gui-component.
     */
    public void addGUIComponent(String id, JComponent component)
    {
        // all GraffitiContainers should be JComponents
        JComponent container = (JComponent) guiMap.get(id);

        if(container != null)
        {
            // if the component is itself a container, then add it to the guiMap
            if(component instanceof GraffitiContainer)
            {
                GraffitiContainer con = (GraffitiContainer) component;
                guiMap.put(con.getId(), con);
            }

            // if the component is a ToolButton, the represented tool has to be
            // added to the mode this ToolButton likes to be in
            if(component instanceof ToolButton)
            {
                ToolButton tb = (ToolButton) component;
                modeManager.getMode(tb.getPreferredComponent()).addTool(tb.getTool());
                tb.addActionListener(tl);

                // if the tool provides undo information the undoSupport has 
                // to be set.
                if(tb.getTool() instanceof AbstractUndoableTool)
                {
                    ((AbstractUndoableTool) tb.getTool()).setUndoSupport(undoSupport);

                    //                        assert (getActiveEditorSession() != null);
                    //                        logger.info("tests if Active Session is null in addGuiComponent() in MainFrame");
                    //                    ((AbstractUndoableTool) tb.getTool()).setSession(getActiveEditorSession());
                }
            }
            else if(component instanceof GraffitiComponent)
            {
                GraffitiComponent gComp = ((GraffitiComponent) component);

                gComp.setMainFrame(this);

                if(component instanceof ViewListener)
                {
                    this.viewManager.addViewListener((ViewListener) component);
                }

                if(component instanceof SessionListener)
                {
                    this.sessionListeners.add((SessionListener) component);
                }
            }

            container.add(component);
            container.validate();

            if(container.getParent() instanceof JSplitPane)
            {
                // adjust divider location
                JSplitPane pane = (JSplitPane) container.getParent();
                pane.setResizeWeight(1.0);

                if(pane.getWidth() != 0)
                    pane.setDividerLocation(1 -
                        ((container.getPreferredSize().getWidth() + 10) / pane.getWidth()));

                if(container.getPreferredSize().getWidth() < 20)
                {
                    container.setMinimumSize(new Dimension(160, pane.getHeight()));
                }
                else
                {
                    container.setMinimumSize(container.getPreferredSize());
                }
            }
        }
        else
        {
            // TODO: intelligente Fallunterscheidung bzgl des ?bergebenen 
            // components -  also bei JMenuItem neues Menu anlegen usw
        }
    }

    /**
     * Adds a <code>SelectionListener</code>.
     *
     * @param sl DOCUMENT ME!
     */
    public void addSelectionListener(SelectionListener sl)
    {
        this.selectionListeners.add(sl);

        for(Iterator it = getSessions().iterator(); it.hasNext();)
        {
            try
            {
                EditorSession eSession = (EditorSession) it.next();
                eSession.getSelectionModel().addSelectionListener(sl);
            }
            catch(ClassCastException cce)
            {
                // ok: non-editor sessions have no selection model
            }
        }
    }

    /**
     * Adds the given session to the list of sessions.
     *
     * @param s the new session to add.
     */
    public void addSession(Session s)
    {
        sessions.add(s);

        if(s instanceof EditorSession)
        {
            SelectionModel selModel = new SelectionModel();
            ((EditorSession) s).setSelectionModel(selModel);

            for(Iterator it = selectionListeners.iterator(); it.hasNext();)
            {
                selModel.addSelectionListener((SelectionListener) it.next());
            }

            selModel.add(new Selection(sBundle.getString("activeSelection")));
            selModel.setActiveSelection(sBundle.getString("activeSelection"));
        }
    }

    /**
     * Adds a <code>SessionListener</code>.
     *
     * @param sl DOCUMENT ME!
     */
    public void addSessionListener(SessionListener sl)
    {
        this.sessionListeners.add(sl);
    }

    /**
     * Removes any messages displayed by calls to <code>showMessage</code> or
     * <code>showError</code>.
     */
    public void clearMessages()
    {
        statusBar.clear();
    }

    /**
     * @see java.awt.event.ComponentListener#componentHidden(java.awt.event.ComponentEvent)
     */
    public void componentHidden(ComponentEvent e)
    {
    }

    /**
     * @see java.awt.event.ComponentListener#componentMoved(java.awt.event.ComponentEvent)
     */
    public void componentMoved(ComponentEvent e)
    {
    }

    /**
     * @see java.awt.event.ComponentListener#componentResized(java.awt.event.ComponentEvent)
     */
    public void componentResized(ComponentEvent e)
    {
        for(Iterator it = activeFrames.iterator(); it.hasNext();)
        {
            GraffitiInternalFrame frame = (GraffitiInternalFrame) it.next();

            if(frame.isMaximum())
            {
                //                frame.setBounds(0, 0, desktop.getWidth(), desktop.getHeight());
                invalidate();

                //                try {
                //                    frame.setMaximum(false);
                //                  frame.setMaximum(true);
                //                } catch (PropertyVetoException pve) {
                //                    // does not matter if not allowed
                //                }
            }
        }
    }

    /**
     * @see java.awt.event.ComponentListener#componentShown(java.awt.event.ComponentEvent)
     */
    public void componentShown(ComponentEvent e)
    {
    }

    /**
     * Creates and adds a new internal frame to the desktop within an existing
     * session.
     *
     * @param viewName a name of the new view
     * @param newFrameTitle the title for the frame, if <code>null</code> or
     *        the empty String no title will be set.
     * @param returnScrollpane DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public JScrollPane createInternalFrame(String viewName,
        String newFrameTitle, boolean returnScrollpane)
    {
        return createInternalFrame(viewName, newFrameTitle, activeSession,
            returnScrollpane);
    }

    /**
     * Creates and adds a new internal frame to the desktop within a new
     * session.
     *
     * @param viewName a name of the new view
     * @param newFrameTitle the title for the frame, if <code>null</code> or
     *        the empty String no title will be set.
     * @param session a new session.
     * @param returnScrollPane DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     *
     * @throws RuntimeException DOCUMENT ME!
     */
    public JScrollPane createInternalFrame(String viewName,
        String newFrameTitle, EditorSession session, boolean returnScrollPane)
    {
        View view;

        try
        {
            view = viewManager.createView(viewName);
            session.getGraph().addAttributeConsumer(view);
            view.setAttributeComponentManager(this.attributeComponentManager);
        }
        catch(Exception e)
        {
            e.printStackTrace();
            JOptionPane.showMessageDialog(this, e.getLocalizedMessage(),
                sBundle.getString("view.notFound"), JOptionPane.ERROR_MESSAGE);

            return null;
        }

        String modeName = uiPrefs.get("standardMode", "org.graffiti.plugins.modes.defaultEditMode");
        session.changeActiveMode(modeManager.getMode(modeName));

        GraffitiInternalFrame frame = null;

        if(!returnScrollPane)
        {
            frame = new GraffitiInternalFrame(session, view, newFrameTitle);
            frame.addInternalFrameListener(gift);
        }

        view.addMessageListener(this);

        if(session == activeSession)
        {
            view.setGraph(activeSession.getGraph());
        }

        ListenerManager lm = session.getGraph().getListenerManager();
        lm.addNonstrictAttributeListener(view);
        lm.addStrictEdgeListener(view);
        lm.addStrictNodeListener(view);
        lm.addStrictGraphListener(view);
        view.setGraph(session.getGraph());

        session.addView(view);
        session.setActiveView(view);

        this.activeSession = (EditorSession) session;

        sessions.add(session);

        if(session instanceof EditorSession)
        {
            SelectionModel selModel = new SelectionModel();
            ((EditorSession) session).setSelectionModel(selModel);

            this.fireSessionChanged(session);

            for(Iterator it = selectionListeners.iterator(); it.hasNext();)
            {
                selModel.addSelectionListener((SelectionListener) it.next());
            }

            selModel.add(new Selection(sBundle.getString("activeSelection")));
            selModel.setActiveSelection(sBundle.getString("activeSelection"));
        }

        //        this.addSession(session);
        JScrollPane scrollPane = new JScrollPane(view.getViewComponent(),
                ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS,
                ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);
        scrollPane.getHorizontalScrollBar().setUnitIncrement(10);
        scrollPane.getVerticalScrollBar().setUnitIncrement(10);
        scrollPane.getViewport().setBackground(Color.WHITE);

        if(!returnScrollPane)
        {
            //        this.fireSessionChanged(session);
            int sizeX = 300;
            int sizeY = 200;
            frame.setPreferredSize(new Dimension(sizeX, sizeY));
            frame.getContentPane().add(scrollPane);
            frame.pack();
            frame.setVisible(true);
            this.desktop.add(frame);

            // maximize view at beginning
            JInternalFrame currentFrame = desktop.getSelectedFrame();

            if((currentFrame == null) || currentFrame.isMaximum())
            {
                try
                {
                    frame.setMaximum(true);
                }
                catch(PropertyVetoException pve)
                {
                    // should not happen
                    throw new RuntimeException(pve);
                }
            }

            viewFrameMapper.put(view, frame);
            activeFrames.add(frame);
        }

        this.zoomListeners.add(view);

        return scrollPane;
    }

    /**
     * Creates and returns a new editor session.
     *
     * @return New session
     */
    public Session createNewSession()
    {
        return new EditorSession();
    }

    /**
     * Disposes the main frame.  Closes all sessions and saves the state (e.g.
     * size and position) of the frame's gui elements.
     */
    public void dispose()
    {
        List l = new LinkedList();

        for(Iterator i = getSessionsIterator(); i.hasNext();)
        {
            Session s = (Session) i.next();
            l.add(s);
        }

        for(Iterator it = l.iterator(); it.hasNext();)
        {
            removeSession((Session) it.next());
        }

        savePreferences();

        System.exit(0);
    }

    /**
     * Informs all <code>SessionListener</code>s that the active session has
     * changed.
     *
     * @param session DOCUMENT ME!
     */
    public void fireSessionChanged(Session session)
    {
        // save activeSession since it holds the currently active tool
        // and will be overwritten when the MainFrame is informed of
        // the sessionChanged event
        Session saveSession = this.activeSession;

        for(Iterator it = this.sessionListeners.iterator(); it.hasNext();)
        {
            ((SessionListener) it.next()).sessionChanged(session);
        }

        // deactivate tool that was last active in the session we are 
        // switching to
        if((session != null) && (session.getLastActiveTool() != null))
        {
            session.getLastActiveTool().deactivate();
        }

        // session we are leaving knows the currently active tool
        if((saveSession != null) && (session != null) &&
            (saveSession.getLastActiveTool() != null))
        {
            saveSession.getLastActiveTool().activate();
        }
    }

    /**
     * Called, if the session or data (except graph data) in the session have
     * been changed.
     *
     * @param session DOCUMENT ME!
     */
    public void fireSessionDataChanged(Session session)
    {
        for(Iterator it = this.sessionListeners.iterator(); it.hasNext();)
        {
            ((SessionListener) it.next()).sessionDataChanged(session);
        }
    }

    /*
     * @see org.graffiti.managers.IOManager.IOManagerListener#inputSerializerAdded(org.graffiti.plugin.io.InputSerializer)
     */
    public void inputSerializerAdded(InputSerializer is)
    {
        updateActions();
    }

    /**
     * Loads a graph from a file;
     *
     * @param file File containing the graph;
     */
    public void loadGraph(File file)
    {
        String fileName = file.getName();
        String ext = fileName.substring(fileName.lastIndexOf("."));

        try
        {
            InputSerializer is = ioManager.createInputSerializer(ext);

            Graph g = is.read(new FileInputStream(file));

            EditorSession es = new EditorSession(g);
            es.setFileName(file.toURI());
            showViewChooserDialog(es, false);
        }
        catch (org.graffiti.plugin.io.ParserException e1) {
        	JOptionPane.showMessageDialog(null, sBundle.getString("fileFormatError").
        			replaceAll("\\[err\\]", e1.getLocalizedMessage()), 
        			sBundle.getString("fileFormatErrorTitle"), JOptionPane.ERROR_MESSAGE);
        }
        catch(Exception e1)
        {
            showError(e1.getLocalizedMessage());
            e1.printStackTrace();
        }
    }

    /*
     * @see org.graffiti.managers.IOManager.IOManagerListener#outputSerializerAdded(org.graffiti.plugin.io.OutputSerializer)
     */
    public void outputSerializerAdded(OutputSerializer os)
    {
        updateActions();
    }

    /**
     * Called by the plugin manager, iff a plugin has been added.
     *
     * @param plugin the added plugin.
     * @param desc the description of the new plugin.
     */
    public void pluginAdded(GenericPlugin plugin, PluginDescription desc)
    {
        try
        {
            EditorPlugin eplugin = (EditorPlugin) plugin;

            Mode[] modes = eplugin.getModes();

            if(modes != null)
            {
                // for every mode create a ModeToolbar
                for(int i = modes.length - 1; i >= 0; i--)
                {
                    ModeToolbar mtb = new ModeToolbar(modes[i]);

                    guiMap.put(mtb.getId(), mtb);
                    leftToolBarPanel.add(mtb);
                    mtb.setVisible(false);
                }
            }

            // add gui component to gui and if necessary update the mapping
            GraffitiComponent[] gc = eplugin.getGUIComponents();

            if(gc != null)
            {
                for(int i = 0; i < gc.length; i++)
                {
                    addGUIComponent(gc[i].getPreferredComponent(),
                        (JComponent) gc[i]);
                }
            }

            Tool[] tools = eplugin.getTools();

            if(tools != null)
            {
                for(int i = tools.length - 1; i >= 0; i--)
                {
                    //            this.zoomListeners.add(tools[i]);
                    //logger.finest("");
                    if(tools[i].isSessionListener())
                    {
                        addSessionListener((SessionListener) tools[i]);
                    }

                    if(tools[i].isViewListener())
                    {
                        viewManager.addViewListener((ViewListener) tools[i]);
                    }

                    if(tools[i].isSelectionListener())
                    {
                        selectionListeners.add(tools[i]);

                        for(Iterator it = sessions.iterator(); it.hasNext();)
                        {
                            Object sess = it.next();

                            if(sess instanceof EditorSession)
                            {
                                if(((EditorSession) sess).getSelectionModel() != null)
                                {
                                    ((EditorSession) sess).getSelectionModel()
                                     .addSelectionListener((SelectionListener) tools[i]);
                                }
                            }

                            // TODO: check what todo if non-EditorSession ...
                        }
                    }
                }
            }
        }
        catch(ClassCastException cce)
        {
            // only EditorPlugins provide Modes / GUIComponents or Tools
        }

        Algorithm[] algorithms = plugin.getAlgorithms();

        // for every algorithm: add a menu entry
        for(int i = algorithms.length - 1; i >= 0; i--)
        {
            Algorithm a = algorithms[i];
            Action action = new RunAlgorithm(a.getClass().getName(),
                    a.getName(), this, editComponentManager, a);

            algorithmActions.add(action);

            JMenuItem item = new JMenuItem(action);
            if (item.getText()!=null)
            	pluginMenu.add(item);
            
            // sort menu items, beginning from the 3rd entry (after the ---)
            // author: c. klukas
            Vector menuItems = new Vector();
            // remove all entries and memorize them
            while (pluginMenu.getItemCount()>2+2) {				// ADDED +2 because of new commands "save and load preferences"
            	menuItems.add(pluginMenu.getItem(2+2));
                pluginMenu.remove(pluginMenu.getItem(2+2));
            }
            // search the first one in the remaining list 
            while (menuItems.size()>0) {
                JMenuItem firstItem = (JMenuItem)menuItems.get(0);
                for (int im=0; im<menuItems.size(); im++) {
                    if (((JMenuItem)menuItems.get(im)).getText().compareToIgnoreCase(firstItem.getText())<=0) {
                        firstItem = (JMenuItem)menuItems.get(im);
                    }
                }
                // add the smalles from the memo-list
                pluginMenu.add(firstItem);
                // remove from memo-list
                menuItems.remove(firstItem);
            }
            // menu items are now in alphabetic list
        }

        // Registers all plugins that are session listeners.
        if(plugin.isSessionListener())
        {
            addSessionListener((SessionListener) plugin);
        }

        if(plugin.isSelectionListener())
        {
            selectionListeners.add(plugin);

            for(Iterator it = sessions.iterator(); it.hasNext();)
            {
                Object sess = it.next();

                if(sess instanceof EditorSession)
                {
                    ((EditorSession) sess).getSelectionModel()
                     .addSelectionListener((SelectionListener) plugin);
                }

                // TODO: check what todo if non-EditorSession ...
            }
        }

        if(plugin.needsEditComponents())
        {
            ((NeedEditComponents) plugin).setEditComponentMap(editComponentManager.getEditComponents());
        }

        updateActions();
    }

    /**
     * Removes a <code>SelectionListener</code>.
     *
     * @param sl DOCUMENT ME!
     */
    public void removeSelectionListener(SelectionListener sl)
    {
        this.selectionListeners.remove(sl);

        for(Iterator it = getSessions().iterator(); it.hasNext();)
        {
            try
            {
                EditorSession eSession = (EditorSession) it.next();
                eSession.getSelectionModel().removeSelectionListener(sl);
            }
            catch(ClassCastException cce)
            {
                // ok: non-editor sessions have no selection model
            }
        }
    }

    /**
     * Closes all views of the given session and removes the session from the
     * list of sessions.
     *
     * @param session the session to be removed.
     */
    public void removeSession(Session session)
    {
        // check if changes have been made
        List views = new LinkedList();

        // close all views and remove this session
        // clone the views list of this session, because it is modified
        // during the iteration and we do not want
        // ConcurrentModificationExceptions
        views.addAll(session.getViews());
        
        for(Iterator i = views.iterator(); i.hasNext();)
        {
            View view = (View) i.next();

            GraffitiInternalFrame frame = (GraffitiInternalFrame) viewFrameMapper.get(view);

            viewFrameMapper.remove(view);
            activeFrames.remove(frame);

            if(frame != null)
            {
                frame.doDefaultCloseAction();
            }

            this.zoomListeners.remove(view);
        }

        sessions.remove(session);
        updateActions();
    }

    /**
     * Removes a <code>SessionListener</code>.
     *
     * @param sl DOCUMENT ME!
     */
    public void removeSessionListener(SessionListener sl)
    {
        this.sessionListeners.remove(sl);
    }

    /**
     * Saves the preferences of the main frame.
     */
    public void savePreferences()
    {
        try
        {
            int screenX = (int) (getLocationOnScreen().getX());
            int screenY = (int) (getLocationOnScreen().getY());

            uiPrefs.putInt("sizeWidth", getWidth());
            uiPrefs.putInt("sizeHeight", getHeight());
            uiPrefs.putInt("positionX", screenX);
            uiPrefs.putInt("positionY", screenY);

            uiPrefs.putInt("vertSplitter", vertSplitter.getDividerLocation());

            uiPrefs.sync();
        }
        catch(IllegalComponentStateException icse)
        {
            icse.printStackTrace();
        }
        catch(BackingStoreException bse)
        {
            bse.printStackTrace(System.out);
        }
    }

    /**
     * Invoked when the session changed.
     *
     * @param s the new session.
     */
    public void sessionChanged(Session s)
    {
        if(isSessionActive())
        {
            Mode oldMode = activeSession.getActiveMode();

            if(oldMode != null)
            {
                ModeToolbar oldtb = (ModeToolbar) guiMap.get(oldMode.getId());
                oldtb.setVisible(false);
            }

            // removing the old sesion from undoSupport
            undoSupport.removeUndoableEditListener(activeSession.getUndoManager());

            // removing the MainFrame from undoSupport
            undoSupport.removeUndoableEditListener(this);
        }

        if(s != null)
        {
            // registering the new session at undoSupport
            undoSupport.addUndoableEditListener(((EditorSession) s).getUndoManager());

            // registering the MainFrame at undoSupport 
            undoSupport.addUndoableEditListener(this);

            // changing the graph in the tools of the new mode
            Mode newMode = s.getActiveMode();
            Tool t = null;

            if(newMode != null)
            {
                List newTools = newMode.getTools();

                for(Iterator it = newTools.iterator(); it.hasNext();)
                {
                    Tool tl = (Tool) it.next();
                    tl.setGraph(s.getGraph());
                }

                ModeToolbar newtb = ((ModeToolbar) (guiMap.get(newMode.getId())));
                newtb.setVisible(true);

                // registering the new activeTool with the views of the new session
                t = newtb.getActiveTool();
            }

            List views = s.getViews();

            for(Iterator it = views.iterator(); it.hasNext();)
            {
                View view = (View) it.next();

                this.zoomListeners.add(view);

                MouseListener[] ml = view.getViewComponent().getMouseListeners();

                //                System.out.println("#MouseListeners: " + ml.length);
                for(int i = ml.length - 1; i >= 0; i--)
                {
                    view.getViewComponent().removeMouseListener(ml[i]);

                    //                    System.out.println("deleting listeners");
                }

                MouseMotionListener[] mml = view.getViewComponent()
                                                .getMouseMotionListeners();

                //                System.out.println("#MouseListeners: " + mml.length);
                for(int i = mml.length; --i >= 0;)
                {
                    view.getViewComponent().removeMouseMotionListener(mml[i]);

                    //                    System.out.println("deleting listeners");
                }

                if(t != null)
                {
                    view.getViewComponent().addMouseListener(t);
                    view.getViewComponent().addMouseMotionListener(t);
                }
            }

            this.activeSession = (EditorSession) s;
            updateActions();
        }
        else
        {
            //            System.out.println("Deactivating tools");
            // TODO what do we have to do here? (jf)
        }
    }

    /**
     * Invoked when the session data changed.
     *
     * @param s DOCUMENT ME!
     */
    public void sessionDataChanged(Session s)
    {
        EditorSession es = (EditorSession) s;

        for(Iterator i = es.getViews().iterator(); i.hasNext();)
        {
            View view = (View) i.next();

            this.zoomListeners.add(view);

            GraffitiInternalFrame frame = (GraffitiInternalFrame) viewFrameMapper.get(view);

            if(es != null)
            {
                if(es.getFileNameAsString() != null)
                {
                    if(frame != null)
                    {
                        frame.setTitle(es.getFileNameAsString());
                    }
                }
            }
        }
        
        updateActions();
    }

    /**
     * Method <code>showMesssage</code> displays a message on GUI components
     * according to the specified type. The message will be displayed for some
     * defined number of seconds.
     *
     * @param message a message string to be displayed
     * @param type a type of the message (e.g. MessageListener.INFO)
     */
    public void showMesssage(String message, int type)
    {
        if(type == MessageListener.ERROR)
        {
            this.statusBar.showError(message);
        }
        else if(type == MessageListener.INFO)
        {
            this.statusBar.showInfo(message);
        }
        else if(type == MessageListener.PERMANENT_INFO)
        {
            this.statusBar.showInfo(message, Integer.MAX_VALUE);
        }
    }

    /**
     * Method <code>showMesssage</code> displays a message on GUI components
     * according to the specified type for the given interval.
     *
     * @param message a message string to be displayed
     * @param type a type of the message (e.g. ERROR)
     * @param timeMillis number of milliseconds the message should be displayed
     */
    public void showMesssage(String message, int type, int timeMillis)
    {
        if(type == MessageListener.ERROR)
        {
            this.statusBar.showError(message, timeMillis);
        }
        else if(type == MessageListener.INFO)
        {
            this.statusBar.showInfo(message, timeMillis);
        }
    }

    /**
     * Method <code>showViewChooserDialog </code> invokes a view chooser dialog
     * for choosing view types. The parameter withNewSession  specifies
     * whether the new view starts within an existing session or within a new
     * session.
     *
     * @param returnScrollpane DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public JScrollPane showViewChooserDialog(boolean returnScrollpane)
    {
        return showViewChooserDialog(activeSession, returnScrollpane);
    }

    /**
     * Method <code>showViewChooserDialog </code> invokes a view chooser dialog
     * for choosing view types. The parameter withNewSession  specifies
     * whether the new view starts within an existing session or within a new
     * session.
     *
     * @param session the session in which to open the new view.
     * @param returnScrollPane DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    public JScrollPane showViewChooserDialog(EditorSession session,
        boolean returnScrollPane)
    {
        String[] views = viewManager.getViewNames();

        if(views.length == 0)
        {
            JOptionPane.showMessageDialog(this,
                sBundle.getString("viewchooser.pluginNotAdded"),
                sBundle.getString("viewchooser.errorDialog.title"),
                JOptionPane.ERROR_MESSAGE);
        }
        else if(viewManager.getViewNames().length == 1)
        {
            if(sessions.contains(session))
            {
                return createInternalFrame(views[0],
                    session.getFileNameAsString(), returnScrollPane);
            }
            else
            {
                return createInternalFrame(views[0],
                    session.getFileNameAsString(), session, returnScrollPane);
            }
        }
        else
        {
            ViewTypeChooser viewChooser = new ViewTypeChooser(this,
                    sBundle.getString("viewchooser.title"),
                    viewManager.getViewNames());

            // The user did not select a view.
            if(viewChooser.getSelectedView() == -1)
            {
                return null;
            }

            String selectedView = views[viewChooser.getSelectedView()];

            if(selectedView != null)
            {
                if(sessions.contains(session))
                {
                    return createInternalFrame(selectedView,
                        session.getFileNameAsString(), returnScrollPane);
                }
                else
                {
                    return createInternalFrame(selectedView,
                        session.getFileNameAsString(), session, returnScrollPane);
                }
            }
        }

        return null;
    }

    /**
     * This method is called when an undoableEdit happened.
     *
     * @see javax.swing.event.UndoableEditListener
     */
    public void undoableEditHappened(UndoableEditEvent e)
    {
        editUndo.update();
        editRedo.update();
    }

    /**
     * Updates the state of the actions.
     */
    public void updateActions()
    {
        newGraph.update();

        fileOpen.update();
        fileClose.update();
        fileSave.update();
        fileSaveAs.update();
        fileSaveAll.update();
        viewNew.update();
        editSelectAll.update();
        
        editUndo.update();
        editRedo.update();

        for(Iterator i = algorithmActions.iterator(); i.hasNext();)
        {
            GraffitiAction action = (GraffitiAction) i.next();
            action.update();
        }
    }

    /*
     * @see org.graffiti.managers.ViewManager.ViewManagerListener#viewTypeAdded(java.lang.String)
     */
    public void viewTypeAdded(String viewType)
    {
        updateActions();
    }

    /**
     * Invoked when the Window is set to be the active Window.
     *
     * @param e the <code>WindowEvent</code> to be processed.
     */
    public void windowActivated(WindowEvent e)
    {
    }

    /**
     * Invoked when a window has been closed as the result of calling  dispose
     * on the window.
     *
     * @param e the <code>WindowEvent</code> to be processed.
     */
    public void windowClosed(WindowEvent e)
    {
    }

    /**
     * Invoked when the user attempts to close the window from the window's
     * system menu.
     *
     * @param e the <code>WindowEvent</code> to be processed.
     */
    public void windowClosing(WindowEvent e)
    {
        dispose();
    }

    /**
     * Invoked when a Window is no longer the active Window.
     *
     * @param e the <code>WindowEvent</code> to be processed.
     */
    public void windowDeactivated(WindowEvent e)
    {
    }

    /**
     * Invoked when a window is changed from a minimized to a normal state.
     *
     * @param e the <code>WindowEvent</code> to be processed.
     */
    public void windowDeiconified(WindowEvent e)
    {
    }

    /**
     * Invoked when a window is changed from a normal to a minimized state.
     *
     * @param e the <code>WindowEvent</code> to be processed.
     */
    public void windowIconified(WindowEvent e)
    {
    }

    /**
     * Invoked the first time a window is made visible.
     *
     * @param e the <code>WindowEvent</code> to be processed.
     */
    public void windowOpened(WindowEvent e)
    {
    }

    /**
     * Shows an error in a modal dialog box.
     *
     * @param msg the message to be shown.
     */
    protected void showError(String msg)
    {
        JOptionPane.showMessageDialog(this, msg,
            StringBundle.getInstance().getString("message.dialog.title"),
            JOptionPane.ERROR_MESSAGE);
    }

    /**
     * Returns the active <code>Tool</code>.
     *
     * @return the active <code>Tool</code>.
     */
    Tool getActiveTool()
    {
        if(activeSession != null)
        {
            return ((ModeToolbar) guiMap.get(activeSession.getActiveMode()
                                                          .getId())).getActiveTool();
        }

        return null;
    }

    /**
     * Method fireViewChanged.
     *
     * @param newView
     */
    void fireViewChanged(View newView)
    {
        activeSession.setActiveView(newView);
        viewManager.viewChanged(newView);
    }

    /**
     * Sets the accel key of the given item.  The accel key information is
     * gathered from the <code>sBundle</code>.
     *
     * @param item DOCUMENT ME!
     * @param action DOCUMENT ME!
     */
    private void setAccelKey(JMenuItem item, GraffitiAction action)
    {
        String accel = sBundle.getString("menu." + action.getName() + ".accel");

        if(accel != null)
        {
            try
            {
                int mask = 0;

                if(accel.startsWith("CTRL"))
                {
                    mask += ActionEvent.CTRL_MASK;
                    accel = accel.substring(5);
                }

                if(accel.startsWith("SHIFT"))
                {
                    mask += ActionEvent.SHIFT_MASK;
                    accel = accel.substring(6);
                }

                if(accel.startsWith("ALT"))
                {
                    mask += ActionEvent.ALT_MASK;
                    accel = accel.substring(4);
                }

                int key = KeyEvent.class.getField("VK_" + accel).getInt(null);
                item.setAccelerator(KeyStroke.getKeyStroke(key, mask));
            }
            catch(Exception e)
            {
                // ignore
                e.printStackTrace();
            }
        }
    }

    /**
     * Creates the action instances.
     */
    private void createActions()
    {
        newGraph = new FileNewAction(this, viewManager);
        fileOpen = new FileOpenAction(this, ioManager, viewManager, sBundle);
        fileSave = new FileSaveAction(this, ioManager, this);
        fileClose = new FileCloseAction(this);

        fileSaveAs = new FileSaveAsAction(this, ioManager, this, sBundle);
        fileSaveAll = new FileSaveAllAction(this, ioManager);

        fileExit = new ExitAction(this);

        viewNew = new ViewNewAction(this, sBundle);

        pluginManagerEdit = new PluginManagerEditAction(this, pluginmgr);

        editUndo = new EditUndoAction(this);
        editRedo = new EditRedoAction(this);

        editCut = new CutAction(this);
        editCopy = new CopyAction(this);
        editPaste = new PasteAction(this);

        editDelete = new DeleteAction(this);
        editSelectAll = new SelectAllAction(this);

        redrawView = new RedrawViewAction(this);
    }

    /**
     * Constructs a menu, and returns the menu.  &quot;menu.&quot;<tt>name</tt>
     * is read from the string bundle.
     * &uqot;menu.&quot;<tt>name</tt>&quot;.icon&quot; is read from the image
     * bundle.
     *
     * @param name the name of the menu item.
     *
     * @return DOCUMENT ME!
     */
    private JMenu createMenu(String name)
    {
        JMenu menu = new JMenu(sBundle.getString("menu." + name));

        guiMap.put("menu." + name, menu);

        try
        {
            String mnem = sBundle.getString("menu." + name + ".mnemonic");

            if(mnem != null)
            {
                menu.setMnemonic(Class.forName("java.awt.event.KeyEvent")
                                      .getField(mnem).getInt(null));
            }
        }
        catch(Exception e)
        {
        }

        return menu;
    }

    /**
     * Creates and returns the menu bar.
     *
     * @return the menu bar.
     */
    private JMenuBar createMenuBar()
    {
        JMenuBar menuBar = new JMenuBar();
        guiMap.put("menu", menuBar);

        // menu for file operations
        JMenu fileMenu = createMenu("file");
        menuBar.add(fileMenu);

        fileMenu.add(createMenuItem(newGraph));
        fileMenu.add(createMenuItem(viewNew));
        fileMenu.add(createMenuItem(fileOpen));
        fileMenu.addSeparator();
        fileMenu.add(createMenuItem(fileSave));
        fileMenu.add(createMenuItem(fileSaveAs));

        //        fileMenu.add(createMenuItem(fileSaveAll)); TODO
        fileMenu.addSeparator();
        fileMenu.add(createMenuItem(fileClose));
        fileMenu.add(createMenuItem(fileExit));

        JMenu editMenu = createMenu("edit");
        menuBar.add(editMenu);

        editMenu.add(createMenuItem(editUndo));
        editMenu.add(createMenuItem(editRedo));
        editMenu.addSeparator();
        editMenu.add(createMenuItem(editCut));
        editMenu.add(createMenuItem(editCopy));
        editMenu.add(createMenuItem(editPaste));
        editMenu.addSeparator();
        editMenu.add(createMenuItem(editDelete));
        editMenu.add(createMenuItem(editSelectAll));
        editMenu.addSeparator();
        editMenu.add(createMenuItem(redrawView));

        pluginMenu = createMenu("plugin");

        menuBar.add(pluginMenu);

        pluginMenu.add(createMenuItem(pluginManagerEdit));
        
        // NEW SAVE / LOAD PREFRENCES ***************************************
        JMenuItem pluginPrefsSave=new JMenuItem("Save Preferences...");
        JMenuItem pluginPrefsLoad=new JMenuItem("Load Preferences...");
        pluginPrefsLoad.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				JFileChooser fc = new JFileChooser();
				fc.showOpenDialog(null);
			    File selFile = fc.getSelectedFile();
			    String err=null;
			    try {
			    	Preferences.importPreferences(new FileInputStream(selFile));
				} catch (FileNotFoundException e1) {
					err=e1.getLocalizedMessage();
				} catch (IOException e1) {
					err=e1.getLocalizedMessage();
				} catch (InvalidPreferencesFormatException e1) {
					err=e1.getLocalizedMessage();
				}
				if (err!=null)
					JOptionPane.showMessageDialog(null, 
							"Error while reading preferences: "+err,
							"Error",
							JOptionPane.ERROR_MESSAGE);
				try {
					getPluginManager().loadStartupPlugins();
				} catch (PluginManagerException e2) {
					String errm = e2.getLocalizedMessage();
					JOptionPane.showMessageDialog(null, 
							"Error while loading plugins: "+errm,
							"Error",
							JOptionPane.ERROR_MESSAGE);
				}
			}});
        pluginPrefsSave.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				JFileChooser fc = new JFileChooser();
				fc.showSaveDialog(null);
			    File selFile = fc.getSelectedFile();
			    String err=null;
			    try {
			    	Preferences prefs = Preferences.userNodeForPackage(GraffitiEditor.class);
			    	prefs.exportSubtree(new FileOutputStream(selFile));
				} catch (FileNotFoundException e1) {
					err=e1.getLocalizedMessage();
				} catch (IOException e1) {
					err=e1.getLocalizedMessage();
				} catch (BackingStoreException e1) {
					err=e1.getLocalizedMessage();
				}
				if (err!=null)
					JOptionPane.showMessageDialog(null, 
							"Error while saving preferences: "+err,
							"Error",
							JOptionPane.ERROR_MESSAGE);
			}});
        pluginMenu.add(pluginPrefsSave);
        pluginMenu.add(pluginPrefsLoad);
        
        // ******************************************************************
        
        pluginMenu.addSeparator();

        windowMenu = createMenu("window");
        menuBar.add(windowMenu);

        return menuBar;
    }
    
    /**
     * Shows an arbitrary message dialog.
     *
     * @param msg the message to be shown.
     */
    public void showMessageDialog(String msg)
    {
        showMessageDialog(msg, 
        		StringBundle.getInstance().getString("message.dialog.title"));
    }    
    
    /**
     * Shows an arbitrary message dialog.
     *
     * @param msg the message to be shown.
     */
    public void showMessageDialog(String msg, String title)
    {
        JOptionPane.showMessageDialog(this, msg,
            title,
            JOptionPane.WARNING_MESSAGE);
    }        

    /**
     * Constructs a menu item, registers this class as action listener and
     * returns the menu item.  &quot;menu.&quot;<tt>name</tt> is read from the
     * string bundle. &uqot;menu.&quot;<tt>name</tt>&quot;.icon&quot; is read
     * from the image bundle.
     *
     * @param action the action, which should be executed by this menu item.
     *
     * @return DOCUMENT ME!
     */
    private JMenuItem createMenuItem(GraffitiAction action)
    {
        String actionName = action.getName();

        JMenuItem item = new JMenuItem(action);
        item.setText(sBundle.getString("menu." + actionName));
        item.setIcon(iBundle.getImageIcon("menu." + actionName + ".icon"));

        try
        {
            String mnem = sBundle.getString("menu." + actionName + ".mnemonic");

            if(mnem != null)
            {
                item.setMnemonic(Class.forName("java.awt.event.KeyEvent")
                                      .getField(mnem).getInt(null));
            }
        }
        catch(Exception e)
        {
            e.printStackTrace();
        }

        setAccelKey(item, action);

        return item;
    }

    /**
     * Creates the editor's tool bar.
     *
     * @return the toolbar for the editor.
     */
    private JToolBar createToolBar()
    {
        JToolBar toolBar = new JToolBar();
        toolBar.setFloatable(false);

        toolBar.add(createToolBarButton(newGraph));
        toolBar.add(createToolBarButton(fileOpen));
        toolBar.addSeparator();
        toolBar.add(createToolBarButton(fileSave));
        toolBar.add(createToolBarButton(fileSaveAs));

        //              toolBar.add(createToolBarButton(fileSaveAll)); TODO
        toolBar.addSeparator();
        toolBar.add(createToolBarButton(editCut));
        toolBar.add(createToolBarButton(editCopy));
        toolBar.add(createToolBarButton(editPaste));
        toolBar.addSeparator();
        toolBar.add(createToolBarButton(editUndo));
        toolBar.add(createToolBarButton(editRedo));
        toolBar.addSeparator();
        toolBar.add(createToolBarButton(pluginManagerEdit));
        toolBar.addSeparator();

        return toolBar;
    }

    /**
     * Constructs and returns a button.
     *
     * @param action the action, which is associated with this button.
     *
     * @return DOCUMENT ME!
     */
    private JButton createToolBarButton(GraffitiAction action)
    {
        JButton button = new JButton(action);
        button.setText(sBundle.getString("toolbar." + action.getName()));
        button.setToolTipText(sBundle.getString("toolbar." + action.getName() +
                ".tooltip"));
        button.setIcon(iBundle.getImageIcon("toolbar." + action.getName() +
                ".icon"));

        //a little bag of tricks: java developers use a string value for this property
        // instead of a constant, moreover is this string value not documented.ww 
        button.putClientProperty("hideActionText", new Boolean(true));

        try
        {
            String mnem = sBundle.getString("toolbar." + action.getName() +
                    ".mnemonic");

            if(mnem != null)
            {
                button.setMnemonic(Class.forName("java.awt.event.KeyEvent")
                                        .getField(mnem).getInt(null));
            }
        }
        catch(Exception e)
        {
            e.printStackTrace();
        }

        return button;
    }

    //~ Inner Classes ==========================================================

    /**
     * Listener for the internal frames.
     */
    class GraffitiInternalFrameListener
        extends InternalFrameAdapter
    {
        /**
         * @see javax.swing.event.InternalFrameListener#internalFrameActivated(InternalFrameEvent)
         */
        public void internalFrameActivated(InternalFrameEvent e)
        {
            super.internalFrameActivated(e);

            GraffitiInternalFrame iframe = (GraffitiInternalFrame) e.getInternalFrame();
            EditorSession frameSession = iframe.getSession();

            if(!(frameSession == activeSession))
            {
                fireSessionChanged(frameSession);
            }
            else
            {
                fireViewChanged(iframe.getView());
            }
        }

        /**
         * @see javax.swing.event.InternalFrameListener#internalFrameClosed(InternalFrameEvent)
         */
        public void internalFrameClosed(InternalFrameEvent e)
        {
            GraffitiInternalFrame f = (GraffitiInternalFrame) e.getInternalFrame();

            EditorSession session = ((GraffitiInternalFrame) e.getInternalFrame()).getSession();

            // remove this view only if there are other open views
            if(session.getViews().size() >= 2)
            {
                f.getView().close();
                session.removeView(f.getView());
                viewFrameMapper.remove(f.getView());
                activeFrames.remove(f);
                zoomListeners.remove(f.getView());
            }
            else
            {
                // remove the session if we are closing the last view
                f.getView().close();

                session.setClosing();
                session.close();
                sessions.remove(session);
                session = null;

                viewFrameMapper.remove(f.getView());
                activeFrames.remove(f);

                if(sessions.size() == 0)
                {
                    //                    System.out.println("last session closed.");
                    fireSessionChanged(null);
                    activeSession = null;
                }
            }

            // the update mechanism after the close of the last session
            // still has some bugs...
            updateActions();

            super.internalFrameClosed(e);
        }
		/* (non-Javadoc)
		 * @see javax.swing.event.InternalFrameListener#internalFrameClosing(javax.swing.event.InternalFrameEvent)
		 */
		public void internalFrameClosing(InternalFrameEvent e) {
			GraffitiInternalFrame f = (GraffitiInternalFrame) e.getInternalFrame();
			EditorSession session = ((GraffitiInternalFrame) e.getInternalFrame()).getSession();
			
			if (session.isModified()) {
				int res =
					JOptionPane.showConfirmDialog(GraffitiSingleton.getInstance().getMainFrame(),
							sBundle.getString("frame.close_save"), sBundle.getString("frame.close_save_title"), 
							JOptionPane.YES_NO_OPTION);
				if (res==JOptionPane.YES_OPTION) {
					// save current graph
					fileSaveAs.actionPerformed(new ActionEvent(e.getSource(), 0, null));
				}
				// continue, close view/session
			}
			super.internalFrameClosing(e);
		}

    }

    /**
     * Special implementation of an ActionListener used for ToolButtons. When
     * the tool changes, the old one is removed as MouseInputListener from the
     * views, and the new one is added.
     */
    class ToolListener
        implements ActionListener
    {

        /**
         * DOCUMENT ME!
         *
         * @param ae DOCUMENT ME!
         */
        public void actionPerformed(ActionEvent ae)
        {
			Tool oldActiveTool = activeSession.getLastActiveTool();
			
            if(oldActiveTool != null) {
                oldActiveTool.deactivate();
            }

            Tool newActiveTool = getActiveTool();
            newActiveTool.activate();

            List views = activeSession.getViews();

            for (Iterator it = views.iterator(); it.hasNext();) {
                View view = (View) it.next();

				view.getViewComponent().removeMouseListener(oldActiveTool);
				view.getViewComponent().removeMouseMotionListener(oldActiveTool);

                view.getViewComponent().addMouseListener(newActiveTool);
                view.getViewComponent().addMouseMotionListener(newActiveTool);
            }
        }
    }
}

//------------------------------------------------------------------------------
//   end of file
//------------------------------------------------------------------------------
