//==============================================================================
//
//   ParameterEditPanel.java
//
//   Copyright (c) 2001-2003 Graffiti Team, Uni Passau
//
//==============================================================================
// $Id: ParameterEditPanel.java,v 1.3 2004/03/31 13:20:27 klukas Exp $

package org.graffiti.editor.dialog;

import java.awt.BorderLayout;
import java.awt.Dimension;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import javax.swing.BoxLayout;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTextField;

import org.graffiti.plugin.ToolTipHelper;
import org.graffiti.plugin.editcomponent.StandardValueEditComponent;
import org.graffiti.plugin.editcomponent.ValueEditComponent;
import org.graffiti.plugin.parameter.Parameter;
import org.graffiti.plugin.parameter.SelectionParameter;

import org.graffiti.selection.Selection;

import org.graffiti.util.InstanceCreationException;
import org.graffiti.util.InstanceLoader;

/**
 * Represents a parameter edit panel.
 *
 * @version $Revision: 1.3 $
 */
public class ParameterEditPanel
    extends JPanel
{
    //~ Instance fields ========================================================

    /** Holds the edit fields cells. */
    private JPanel editFieldPanel;

    /** Holds the id cells. */
    private JPanel idPanel;

    /** DOCUMENT ME! */
    private JScrollPane tableScroll;

    /** Splits table into id and edit field. */
    private JSplitPane tableSplit;

    /** DOCUMENT ME! */
    private List displayedVEC;

    /**
     * Maps from an displayable class name to the class name of a
     * <code>ValueEditComponent</code>.
     */
    private Map editTypeMap;

    /** The list of parameters to display and edit. */
    private Parameter[] parameters;

    //~ Constructors ===========================================================

    /**
     * Instantiates a new edit panel.
     *
     * @param parameters DOCUMENT ME!
     * @param editTypes DOCUMENT ME!
     * @param selection DOCUMENT ME!
     */
    public ParameterEditPanel(Parameter[] parameters, Map editTypes,
        Selection selection)
    {
        super();

        this.parameters = parameters;

        this.displayedVEC = new LinkedList();
        setEditTypeMap(editTypes);

        idPanel = new JPanel();
        idPanel.setLayout(new BoxLayout(idPanel, BoxLayout.Y_AXIS));

        editFieldPanel = new JPanel();
        editFieldPanel.setLayout(new BoxLayout(editFieldPanel, BoxLayout.Y_AXIS));

        tableSplit = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, idPanel,
                editFieldPanel);
        tableSplit.setDividerLocation(120); // 75
        tableSplit.setDividerSize(3);
        tableSplit.setOneTouchExpandable(true);

        tableScroll = new JScrollPane(tableSplit);
        // tableScroll.setPreferredSize(new Dimension(200, 200));

        this.setLayout(new BorderLayout());
        this.add(tableScroll, BorderLayout.CENTER);

        buildTable(selection);

        this.revalidate();
    }

    //~ Methods ================================================================

    /**
     * Sets the map of displayable types to the given map.
     *
     * @param map DOCUMENT ME!
     */
    public void setEditTypeMap(Map map)
    {
        this.editTypeMap = map;
    }

    /**
     * Sets the paramter array this panel displays.
     *
     * @param params
     */
    public void setParameters(Parameter[] params)
    {
        this.parameters = params;
    }

    /**
     * Returns the array of parameters with the values updated from the dialog.
     *
     * @return Parameter[]
     */
    public Parameter[] getUpdatedParameters()
    {
        for(Iterator it = displayedVEC.iterator(); it.hasNext();)
        {
            ((ValueEditComponent) it.next()).setValue();
        }

        return this.parameters;
    }

    /**
     * Builds the table that is used for editing parameters
     *
     * @param selection list of parameters.
     */
    public void buildTable(Selection selection)
    {
        idPanel.removeAll();
        editFieldPanel.removeAll();
        displayedVEC = new LinkedList();
        addValueEditComponents(idPanel, editFieldPanel, selection);
        revalidate();
    }

    /**
     * Returns a (noneditable) textfield showing the value of the
     * <code>toString</code> method of the parameter.  Used when there is no
     * registered <code>ValueEditComponent</code>.
     *
     * @param parameter DOCUMENT ME!
     *
     * @return DOCUMENT ME!
     */
    private ValueEditComponent getStandardEditComponent(Parameter parameter)
    {
        ValueEditComponent vec = new StandardValueEditComponent(parameter);

        //        JTextField textField = new JTextField(parameter.getValue().toString());
        JTextField textField = (JTextField) vec.getComponent();
        textField.setEditable(false);
        textField.setMinimumSize(new Dimension(0, 20));
        textField.setPreferredSize(new Dimension(100, 30));
        textField.setMaximumSize(new Dimension(2000, 40));

        return vec;
    }

    /**
     * Add one row in the panel.
     *
     * @param idPanel
     * @param editFieldPanel
     * @param parameter
     * @param ecClass
     *
     * @throws RuntimeException DOCUMENT ME!
     */
    private void addRow(JPanel idPanel, JPanel editFieldPanel,
        Parameter parameter, Class ecClass)
    {
        JTextField textField = new JTextField(parameter.getName());
        textField.setToolTipText(parameter.getDescription());
        textField.setHorizontalAlignment(JTextField.RIGHT);
        textField.setEditable(false);

        ValueEditComponent editComp = null;

        try
        {
            editComp = (ValueEditComponent) InstanceLoader.createInstance(ecClass,
                    "org.graffiti.plugin.Displayable", parameter);
            ToolTipHelper.addToolTip(editComp.getComponent(), parameter.getDescription());
        }
        catch(InstanceCreationException ice)
        {
            throw new RuntimeException(
                "Could not create an instance of a ValueEditComponent class. " +
                ice);
        }

        editComp.setDisplayable(parameter);
        editComp.setEditFieldValue();

        JComponent editCompComp = editComp.getComponent();
        textField.setMinimumSize(new Dimension(0,
                (int) editCompComp.getMinimumSize().getHeight()));
        textField.setPreferredSize(editCompComp.getPreferredSize());
        textField.setMaximumSize(new Dimension(2000,
                (int) editCompComp.getMaximumSize().getHeight()));
        idPanel.add(textField);
        editFieldPanel.add(editCompComp);
        displayedVEC.add(editComp);
    }

    /**
     * Add one row in the panel using a standard edit component.
     *
     * @param parameter
     * @param idPanel
     * @param editFieldPanel
     */
    private void addStandardRow(Parameter parameter, JPanel idPanel,
        JPanel editFieldPanel)
    {
        ValueEditComponent editComp = getStandardEditComponent(parameter);
        displayedVEC.add(editComp);

        JTextField textField;
        textField = new JTextField(parameter.getName());
        textField.setToolTipText(parameter.getDescription());
        textField.setHorizontalAlignment(JTextField.RIGHT);
        textField.setEditable(false);

        JComponent editCompComp = editComp.getComponent();
        editCompComp.setToolTipText(parameter.getDescription());
        textField.setMinimumSize(new Dimension(0,
                (int) editCompComp.getMinimumSize().getHeight()));
        textField.setMaximumSize(new Dimension(2000,
                (int) editCompComp.getMaximumSize().getHeight()));
        idPanel.add(textField);
        editFieldPanel.add(editCompComp);
    }

    /**
     * Puts text fields for the IDs in a panel.
     *
     * @param idPanel the list of parameters
     * @param editFieldPanel DOCUMENT ME!
     * @param selection DOCUMENT ME!
     */
    private void addValueEditComponents(JPanel idPanel, JPanel editFieldPanel,
        Selection selection)
    {
        for(int i = 0; i < parameters.length; i++)
        {
            if(parameters[i] instanceof SelectionParameter)
            {
                // use currently active (given) selection instead
                parameters[i] = new SelectionParameter(parameters[i].getName(),
                        parameters[i].getDescription());
                parameters[i].setValue(selection);
            }

            /*
             * check whether there exists a ValueEditComponent, if not use
             * standard edit component */
            Class ecClass = (Class) this.editTypeMap.get(parameters[i].getClass());

            if(ecClass != null)
            {
                // if we have a registered component to display it, add it
                this.addRow(idPanel, editFieldPanel, parameters[i], ecClass);
            }
            else
            {
                // no component registered for this basic displayable
                addStandardRow(parameters[i], idPanel, editFieldPanel);
            }
        }
    }
}

//------------------------------------------------------------------------------
//   end of file
//------------------------------------------------------------------------------
