//==============================================================================
//
//   AttributeListener.java
//
//   Copyright (c) 2001-2003 Graffiti Team, Uni Passau
//
//==============================================================================
// $Id: AttributeListener.java,v 1.2 2003/08/02 09:19:29 holleis Exp $

package org.graffiti.event;

/**
 * Interfaces an attribute events listener.
 *
 * @version $Revision: 1.2 $
 *
 * @see AttributeEvent
 */
public interface AttributeListener
    extends TransactionListener
{
    //~ Methods ================================================================

    /**
     * Called after an attribute has been added.
     *
     * @param e the AttributeEvent detailing the changes.
     */
    void postAttributeAdded(AttributeEvent e);

    /**
     * Called after an attribute has been changed.
     *
     * @param e the AttributeEvent detailing the changes.
     */
    void postAttributeChanged(AttributeEvent e);

    /**
     * Called after an attribute has been removed.
     *
     * @param e the AttributeEvent detailing the changes.
     */
    void postAttributeRemoved(AttributeEvent e);

    /**
     * Called just before an attribute is added.
     *
     * @param e the AttributeEvent detailing the changes.
     */
    void preAttributeAdded(AttributeEvent e);

    /**
     * Called before a change of an attribute takes place.
     *
     * @param e the AttributeEvent detailing the changes.
     */
    void preAttributeChanged(AttributeEvent e);

    /**
     * Called just before an attribute is removed.
     *
     * @param e the AttributeEvent detailing the changes.
     */
    void preAttributeRemoved(AttributeEvent e);
}

//------------------------------------------------------------------------------
//   end of file
//------------------------------------------------------------------------------
