//==============================================================================
//
//   AbstractGraphElement.java
//
//   Copyright (c) 2001-2003 Graffiti Team, Uni Passau
//
//==============================================================================
// $Id: AbstractGraphElement.java,v 1.2 2003/08/02 09:19:29 holleis Exp $

package org.graffiti.graph;

import java.util.logging.Logger;

import org.graffiti.attributes.AbstractAttributable;
import org.graffiti.attributes.CollectionAttribute;

import org.graffiti.event.ListenerManager;

/**
 * GraphElements are Attributables which know the graph they belong to. This
 * class provides the functionality for accessing the graph.
 *
 * @see AbstractNode
 * @see AbstractEdge
 * @see Node
 * @see Edge
 */
public abstract class AbstractGraphElement
    extends AbstractAttributable
    implements GraphElement
{
    //~ Static fields/initializers =============================================

    /** The logger for the current class. */
    private static final Logger logger = Logger.getLogger(AbstractGraphElement.class.getName());

    //~ Instance fields ========================================================

    /** The graph the current <code>AbstractGraphElement</code> belongs to. */
    protected Graph graph;

    //~ Constructors ===========================================================

    /**
     * Constructs a new <code>AbstrctGraphElement</code>.
     */
    public AbstractGraphElement()
    {
    }

    /**
     * Constructs a new <code>AbstrctGraphElement</code>.
     *
     * @param coll the <code>CollectionAttribute</code> of the new
     *        <code>AbstractGraphElement</code> instance.
     */
    public AbstractGraphElement(CollectionAttribute coll)
    {
        super(coll);
    }

    /**
     * Constructs a new <code>AbstrctGraphElement</code>. Sets the graph of the
     * current <code>AbstrctGraphElement</code>.
     *
     * @param graph the graph the <code>AbstrctGraphElement</code> belongs to.
     * @param coll the <code>CollectionAttribute</code> of the new
     *        <code>AbstractGraphElement</code> instance.
     */
    public AbstractGraphElement(Graph graph, CollectionAttribute coll)
    {
        super(coll);
        this.graph = graph;
    }

    /**
     * Constructs a new <code>AbstrctGraphElement</code>. Sets the graph of the
     * current <code>AbstrctGraphElement</code>.
     *
     * @param graph the graph the <code>AbstrctGraphElement</code> belongs to.
     */
    public AbstractGraphElement(Graph graph)
    {
        assert graph != null;
        this.graph = graph;
    }

    //~ Methods ================================================================

    /**
     * Returns the Graph the <code>AbstractGraphElement</code> belongs to.
     *
     * @return the Graph the GraphElement belongs to.
     */
    public Graph getGraph()
    {
        return this.graph;
    }

    /**
     * Returns the ListenerManager of the <code>GraphElement</code>.
     *
     * @return the ListenerManager of the <code>GraphElement</code>.
     */
    public ListenerManager getListenerManager()
    {
        return this.getGraph().getListenerManager();
    }
}

//------------------------------------------------------------------------------
//   end of file
//------------------------------------------------------------------------------
