//==============================================================================
//
//   DefaultToolManager.java
//
//   Copyright (c) 2001-2003 Graffiti Team, Uni Passau
//
//==============================================================================
// $Id: DefaultToolManager.java,v 1.5 2003/12/03 14:00:05 pick Exp $

package org.graffiti.managers;

import java.util.HashSet;
import java.util.Set;

import org.graffiti.managers.pluginmgr.PluginDescription;
import org.graffiti.plugin.EditorPlugin;
import org.graffiti.plugin.GenericPlugin;
import org.graffiti.plugin.mode.Mode;
import org.graffiti.plugin.tool.Tool;

/**
 * Manages the list of tools.
 *
 * @version $Revision: 1.5 $
 */
public class DefaultToolManager
    implements ToolManager
{
    //~ Instance fields ========================================================

    /** List of all available tools. */
    private Set tools;
    
    /** mode manager */
    private ModeManager modeManager;

    //~ Constructors ===========================================================

    /**
     * Constructs a new tool manager.
     */
    public DefaultToolManager(ModeManager modeManager)
    {
        tools = new HashSet();
        this.modeManager = modeManager;
    }

    //~ Methods ================================================================

    /*
     * @see org.graffiti.managers.ToolManager#addTool(org.graffiti.plugin.tool.Tool)
     */
    public void addTool(Tool tool)
    {
        tools.add(tool);
    }

    /*
     * @see org.graffiti.managers.pluginmgr.PluginManagerListener#pluginAdded(org.graffiti.plugin.GenericPlugin, org.graffiti.managers.pluginmgr.PluginDescription)
     */
    public void pluginAdded(GenericPlugin plugin, PluginDescription desc)
    {
        try
        {
            Tool[] theTools = ((EditorPlugin) plugin).getTools();

            if(theTools != null)
            {
                for(int i = theTools.length; --i >= 0;)
                {
                    this.tools.add(theTools[i]);
                    
                    Mode mm = this.modeManager.getMode("org.graffiti.plugins.modes.defaultEditMode");
                    if (mm != null) {
                        mm.addTool(theTools[i]);
                    } else {
                        // TODO: LOG ERROR
                    }
                }
            }
        }
        catch(ClassCastException cce)
        {
            // only EditorPlugins provide tools
        }
    }
}

//------------------------------------------------------------------------------
//   end of file
//------------------------------------------------------------------------------
