//==============================================================================
//
//   Algorithm.java
//
//   Copyright (c) 2001-2003 Graffiti Team, Uni Passau
//
//==============================================================================
// $Id: Algorithm.java,v 1.2 2003/08/02 09:19:28 holleis Exp $

package org.graffiti.plugin.algorithm;

import org.graffiti.graph.Graph;

import org.graffiti.plugin.parameter.*;

/**
 * An <code>Algorithm</code> has to provide a function that returns an array of
 * <code>Parameters</code> it needs, in order to give user interfaces the
 * possibility to provide the <code>Algorithm</code> with apropriate
 * parameters.
 *
 * @version $Revision: 1.2 $
 */
public interface Algorithm
{
    //~ Methods ================================================================

    /**
     * Returns the name (id) of the algorithm.
     *
     * @return DOCUMENT ME!
     */
    public String getName();

    /**
     * Sets the parameters for this algorithm. Must have the same types and
     * order as the array returned by <code>getParameter</code>.
     */
    public void setParameters(Parameter[] params);

    /**
     * Returns a list of <code>Parameter</code> that are set for this
     * algorithm.
     *
     * @return a collection of <code>Parameter</code> that are needed by the
     *         <code>Algorithm</code>.
     */
    public Parameter[] getParameters();

    /**
     * Attaches the given graph to this algorithm.
     *
     * @param g the graph to attach.
     */
    public void attach(Graph g);

    /**
     * Checks whether all preconditions of the current graph are satisfied.
     *
     * @throws PreconditionException if the preconditions of the current graph
     *         are not satisfied.
     */
    public void check()
        throws PreconditionException;

    /**
     * Executes the whole algorithm.
     */
    public void execute();

    /**
     * Resets the internal state of the algorithm.
     */
    public void reset();
}

//------------------------------------------------------------------------------
//   end of file
//------------------------------------------------------------------------------
