//==============================================================================
//
//   NodeShape.java
//
//   Copyright (c) 2001-2003 Graffiti Team, Uni Passau
//
//==============================================================================
// $Id: NodeShape.java,v 1.2 2003/08/02 09:19:55 holleis Exp $

package org.graffiti.plugin.view;

import java.awt.geom.Line2D;
import java.awt.geom.Point2D;

import org.graffiti.graphics.NodeGraphicAttribute;

/**
 * Interface extending <code>GraphElementShape</code>.  Classes implementing
 * this interface are shapes that are displayed for nodes.   They must
 * implement a method returning the intersection between themselves and a
 * <code>Line2D</code> so that the clipping between the node and incoming /
 * outgoing edges can be calculated.
 */
public interface NodeShape
    extends GraphElementShape
{
    //~ Methods ================================================================

    /**
     * Calculates the intersection point between this node shape and a line.
     * For irregularly shaped objects, the intersection that is nearest to the
     * <b>end point</b> of the line is returned.
     *
     * @param line the line with which the intersection should be calculated.
     *
     * @return the intersection point between this node shape and the line.
     */
    public Point2D getIntersection(Line2D line);

    /**
     * Shapes itself according to the graphics attribute found in the
     * CollectionAttribute <code>graphics</code>
     *
     * @param graphics the <code>CollectionAttribute</code> accoring to which
     *        this shape is constructed.
     */
    public void buildShape(NodeGraphicAttribute graphics)
        throws ShapeNotFoundException;
}

//------------------------------------------------------------------------------
//   end of file
//------------------------------------------------------------------------------
