//==============================================================================
//
//   AddEdgeEdit.java
//
//   Copyright (c) 2001-2003 Graffiti Team, Uni Passau
//
//==============================================================================
// $Id: AddEdgeEdit.java,v 1.6 2003/11/03 12:37:17 forster Exp $

package org.graffiti.undo;

import java.util.Map;

import org.graffiti.graph.Edge;
import org.graffiti.graph.Graph;
import org.graffiti.graph.Node;

/**
 * Class <code>AddNodeEdit</code> makes the add edge action undoable.
 *
 * @author Walter Wirch
 * @version $Revision: 1.6 $
 */
public class AddEdgeEdit
    extends GraphElementsEdit
{
    //~ Instance fields ========================================================

    /** added edge */
    private Edge edge;

    //~ Constructors ===========================================================

    /**
     * Constructor for AddEdgeEdit.
     *
     * @param edge
     * @param graph
     * @param geMap
     */
    public AddEdgeEdit(Edge edge, Graph graph, Map geMap)
    {
        super(graph, geMap);
        this.edge = edge;
    }

    //~ Methods ================================================================

    /**
     * Used to display the name for this edit.
     *
     * @return the name of this edit.
     *
     * @see javax.swing.undo.UndoableEdit
     */
    public String getPresentationName()
    {
        return sBundle.getString("undo.addEdge");
    }

    /*
     * @see org.graffiti.undo.GraffitiAbstractUndoableEdit#execute()
     */
    public void execute()
    {
    }

    /**
     * Adds the same edge that was added through the method that created  this
     * edit.
     */
    public void redo()
    {
        super.redo();

        Node source = (Node) getCurrentGraphElement(edge.getSource());
        Node target = (Node) getCurrentGraphElement(edge.getTarget());
        Edge newEdge = graph.addEdgeCopy(edge, source, target);
        geMap.put(edge, newEdge);
    }

    /**
     * Deletes the edge whose addition is stored in this edit.
     */
    public void undo()
    {
        super.undo();

        edge = (Edge) getCurrentGraphElement(edge);
        graph.deleteEdge(edge);
    }
}

//------------------------------------------------------------------------------
//   end of file
//------------------------------------------------------------------------------
