//==============================================================================
//
//   GraffitiAbstractUndoableEdit.java
//
//   Copyright (c) 2001-2003 Graffiti Team, Uni Passau
//
//==============================================================================
// $Id: GraffitiAbstractUndoableEdit.java,v 1.5 2003/08/14 10:06:07 wirch Exp $

package org.graffiti.undo;

import java.util.Map;

import javax.swing.undo.AbstractUndoableEdit;

import org.graffiti.core.StringBundle;

import org.graffiti.graph.GraphElement;

/**
 * GraffitiAbstractUndoableEdit
 *
 * @version $Revision: 1.5 $
 */
public abstract class GraffitiAbstractUndoableEdit
    extends AbstractUndoableEdit {
    //~ Instance fields ========================================================

    /**
     * The reference for the map between graph elements recreated after undo
     * processing and  original graph elements.
     */
    protected Map geMap;

    /** The <code>StringBundle</code> of the main frame. */
    protected StringBundle sBundle = StringBundle.getInstance();

    //~ Constructors ===========================================================

    /**
     * Creates a new <code>GraffitiAbstractUndoableEdit</code> object.
     *
     * @param geMap reference to the map supports the undo operations.
     */
    public GraffitiAbstractUndoableEdit(Map geMap) {
        super();
        this.geMap = geMap;
    }

    //~ Methods ================================================================

    /**
     * Executes action for corresponding tools.
     */
    public abstract void execute();

    /**
     * Returne a new graph element reference through the mapping from old ones.
     *
     * @param ge a graph element has to be updated.
     *
     * @return a new existing graph element mapped from the given ones.
     */
    protected GraphElement getCurrentGraphElement(GraphElement ge) {
        GraphElement newGE = ge;

        if(geMap.containsKey(ge)) {
            newGE = (GraphElement) geMap.get(ge);

            while(geMap.containsKey(newGE)) {
                newGE = (GraphElement) geMap.get(newGE);
            }
        }

        return newGE;
    }
}

//------------------------------------------------------------------------------
//   end of file
//------------------------------------------------------------------------------
